//
// Created by zx on 2020/7/14.
//

/*
 * 指令管理, 包括指令的接收,指令的执行控制,指令的状态追踪,出入口状态检查
 */
#ifndef NNXX_TESTS_COMMAND_MANAGER_H
#define NNXX_TESTS_COMMAND_MANAGER_H

#include <error_code.h>
#include <terminal_message.pb.h>
#include <communication_message.h>
#include "thread_condition.h"
#include "singleton.h"
#include "TaskQueue/TQFactory.h"
#include "TaskQueue/BaseTask.h"
#include "process_message.pb.h"
#include "thread_safe_map.h"
#include "system_setting.pb.h"

class Command_manager :public Singleton<Command_manager>{
    friend Singleton<Command_manager>;
public:
    ~Command_manager();

    /*
     * 初始化函数,创建线程池,创建状态广播线程,根据配置创建停取车流程
     */
    Error_manager init(setting::System_setting system_setting);

    /*
     * 执行停车请求
     */
    Error_manager execute_store_command(message::Store_command_request_msg& request,message::Store_command_response_msg& response);

    /*
     * 执行取车请求
     */
    Error_manager execute_pickup_command(message::Pickup_command_request_msg& request,message::Pickup_command_response_msg& response);

    /*
     * 更新进度状态
     */
    void erase_statu(message::Storing_process_statu statu);
    void erase_statu(message::Picking_process_statu statu);
    void updata_statu(message::Storing_process_statu statu);
    void updata_statu(message::Picking_process_statu statu);
private:
    Command_manager();
    void publish_step_status();
    static void publish_thread_func(Command_manager* command_manager);

protected:

    tq::IQueue*                     m_thread_queue_process;         //指令流程线程池

    std::mutex                          m_process_statu_lock;
    message::Process_statu_map_msg      m_process_statu_msg;
    std::thread*                    m_publish_statu_thread;             //广播状态线程
    Thread_condition				m_publish_exit_condition;			//发送的条件变量
};


#endif //NNXX_TESTS_COMMAND_MANAGER_H
